<?php
/**
 * Classe per la creazione di form html.
 * Costruisce il form a partire da una definizione in XML, mette a disposizione
 * funzionalita' per la validazione dei dati (client e server side), precompilazione
 * dei campi, popolamento dei campi <select> da database, modifica
 * run-time delle label ed action del form. Consente di replicare porzioni o
 * l'intero form N volte senza duplicazioni nella definizione XML.
 *
 * Version 1.4	- possibilità di utilizzare gli <optgroup> negli elementi <select>
 * 
 * Version 1.3	- modificata struttura html del form
 *				- modificati attributi id, name, class generati automaticamente
 *				- migliorato controllo campi required per elementi ripetuti
 *				- migliorato controllo di validazione per elementi ripetuti
 * 
 * Version 1.2	- corretta la validazione dei campi con prefisso
 * 				- aggiunta formattazione automatica dei decimali
 * 				- migliorata maschera di inserimento e validazione date
 *
 * Version 1.1	- aggiunto plugin TinyMCE
 * 				- caricamento dei JS solo per le estensioni utilizzate
 *
 * @package		FormBuilder
 * @author		Andrea Venturi
 * @copyright	Copyright (c) 2009-2012 Andrea Venturi
 * @license		http://www.gnu.org/licenses/lgpl.html	GNU Lesser General Public License
 * @version		1.4
 */
class FormBuilder
{
	protected $db;
	protected $errors;
	protected $readonly;
	protected $repeat;
	protected $search;
	protected $selects;
	protected $t;
	protected $values;
	protected $view;
	protected $xml;
	protected $uniqid;
	protected $expiration;
	protected $field_required;
	protected $field_validation;

	/**
	 * Costruttore della classe
	 * @param	string	 $file      il percorso completo del file di definizione del form
	 * @param	Database $database  oggetto per la gestione del database
	 * @param	array	 $values    array con i valori predefiniti per ogni campo del form
	 * @return	void
	 */
	public function __construct($file, $database=null, $values=array())
	{
		$this->t        = new Translate();
		$this->db		= $database;
		$this->xml		= simplexml_load_file($file);
		$this->values	= $values;

		$this->uniqid	= md5(uniqid(mt_rand(), true));
		$this->cleanSession();

		$this->errors	= array();
		$this->readonly	= array();
		$this->repeat	= array();
		$this->selects	= array();
		$this->field_required = array();
		$this->field_validation = array();

		// scadenza sessione a 45 secondi
		$this->expiration = 45;
	}

	/**
	 * Si occupa di eliminare i dati del form obsoleti dalla sessione
	 * @return void
	 */
	protected function cleanSession()
	{
//		echo '@@SESSION:<pre>'.print_r($_SESSION[__CLASS__], true)."</pre>@@<br />\n";
		
		if(isset($_SESSION[__CLASS__]))
		{
			foreach($_SESSION[__CLASS__] as $k1=>$v1)
			{
				if( time()>=$v1['expire'] )
				{
					unset($_SESSION[__CLASS__][$k1]);
				}
			}
		}

		// $_SESSION[__CLASS__][$this->uniqid]['readonly']
		// $_SESSION[__CLASS__][$this->uniqid]['required']
		// $_SESSION[__CLASS__][$this->uniqid]['validation']
		// $_SESSION[__CLASS__][$this->uniqid]['expire']
	}

	/**
	 * Imposta la scadenza del token CSRF
	 * @param int $seconds
	 */
	public function setCsrfExp($seconds)
	{
		$this->expiration = $seconds;
	}

	public function setXml($xml)
	{
		$this->xml = simplexml_load_file($xml);
	}

	public function getValues()
	{
		return $this->values;
	}

	/**
	 * Sets default values for the elements in the form.
	 * @param	array	$values	array('field_id_1' => 'value_1', 'field_id_2' => 'value_2')
	 * @return	void
	 */
	public function setValues($values)
	{
		$new = array();
		foreach($values as $k=>$v)
		{
//			$new[strtolower($k)] = $v;
			$new[$k] = $v;
		}
		$this->values = $new;
	}

	/**
	 * Unsets default values for the elements in the form
	 * @return void
	 */
	public function unsetValues()
	{
		unset($this->values);
	}

	/**
	 * Sets additional values to insert inside <select> elements.
	 * @param	array	$values  options displayed in <select> elements. array('field_id_1' => array('key_1_1'=>'value_1_1'))
	 * @return	void
	 */
	public function setSelects($values)
	{
		$this->selects = $values;
	}

	/**
	 * Imposta l'array con il numero di ripetizioni per ogni pagina del form
	 * @param	array	$repeat	array associativo con "id_pagina" => numero ripetizioni
	 * @return	void
	 */
	public function setRepeat($repeat)
	{
		$this->repeat = $repeat;
	}

	public function setReadOnly($readonly)
	{
		$this->readonly	= $readonly;
	}

	public function getReadOnly()
	{
		return $this->readonly;
	}

	protected function loadReadOnly()
	{
		$arr = $this->xml->xpath('//field[readonly=1]');

		foreach($arr as $obj)
		{
			$this->readonly[strval($obj->id)] = 1;
		}
	}

	/**
	 * Sets the attribute "action" in the form
	 * @param	string	$action
	 * @return	void
	 */
	public function setAction($action)
	{
		$this->xml->action = $action;
	}

	/**
	 * Sets the label for an element, given its id
	 * @param	string	$label
	 * @param	int		$id
	 * @return	void
	 */
	public function setLabel($label, $id)
	{
		$res = $this->findId($id);
		if(!empty($res)) $res->label = $label;
	}

	/**
	 * Finds an element (input, select, button, textarea), given its id
	 * @param	string	$id	element's id to retrieve
	 * @return	SimpleXMLElement | false
	 */
	protected function findId($id)
	{
		foreach($this->xml->page as $page)
		{
			foreach($page->section as $section)
			{
				$res = $this->scanSection($section, $id);
				if(!empty($res)) return $res;
			}
		}
		return false;
	}

	/**
	 * Recursive function to find an element (input, select, button, textarea), given its id
	 * @param	SimpleXMLElement	$section	SimpleXMLElement to scan
	 * @param	string				$id			element's id to retrieve
	 * @return SimpleXMLElement or false
	 */
	protected function scanSection($section, $id)
	{
		foreach($section->field as $field)
		{
			if(strval($field->id) == $id) return $field;
		}

		foreach($section->section as $s2)
		{
			$res = $this->scanSection($s2, $id);
			if(isset($res) && !$res) return $res;
		}
		return false;
	}

	/**
	 * Returns an array with all the ids of the fields in the form
	 * @return array
	 */
	public function getIds()
	{
		$arr = $this->xml->xpath('//field/id');
		$arr2 = array();
		foreach($arr as $val)
		$arr2[] = strval($val);

		return $arr2;
	}

	/**
	 * Returns an array with all the ids of the fields in the form and values as default
	 * @param $value default value for fill the array
	 * @return array
	 *
	 public function getFields($value=0)
	 {
		$keys = $this->getIds();

		return array_fill_keys($keys,$value);
		}
		*/
	/**
	 * Valids the values posted with the form
	 * @return bool
	 */
	public function isValid()
	{
		$this->uniqid = $_POST['uniqid'];

		// 20/07/2011 non utilizzo più la sessione per i campi readonly, perché
		// con dalla versione 1.3 vengono postati come campi hidden
		$arr = $_POST;
		
		// 09/10/2009 MODIFICA faccio il merge tra i campi postati e quelli readonly in sessione
		// mi serve per visualizzare i dati nel form senza ricaricarli da DB
/*		if(isset($_SESSION[__CLASS__][$this->uniqid]['readonly']))
		{
			$arr = $_POST;
		}
		else
		{
			$arr = $_POST;
		}
*/		
		
		// controllo del token CSRF
		$tmp = $this->xml->xpath('//field[type="csrf"]');
		
		if( !empty($tmp) && !isset($_SESSION[__CLASS__][$this->uniqid]) )
		{
			return false;
		}
		
		
		/* cancella i dati dalla sessione.
			- se è valido guadagno spazio perchè non servono più
			- se non è valido sono pari xkè poi viene ricostruito comunque
			*/
//		unset($_SESSION[__CLASS__][$this->uniqid]['readonly']);

//		echo '@@<pre>'. print_r($arr, true) ."</pre>@@<br>\n";
		
		// imposto i valori dei campi e faccio controlli
		$arr = Utility::array_flat($arr, '', '-');
		$this->setValues($arr);
		
//		echo '@@<pre>'. print_r($arr, true) ."</pre>@@<br>\n";

		$v1 = $this->checkRequired();
		$v2 = $this->validation();
		
		return ($v1 && $v2);
	}

	/**
	 * Cerca i field con required=1 e controlla che non sia vuoto.
	 * Checks all required fields of the form
	 * @return bool
	 */
	protected function checkRequired()
	{
//		$values = Utility::array_flat($this->values, '', '-');
		$values = $this->values;
		
		if(isset($_SESSION[__CLASS__][$this->uniqid]['required']))
			$session = $_SESSION[__CLASS__][$this->uniqid]['required'];
		else
			$session = null;

		$valid = true;
		if(is_array($session))
		{
			foreach($session as $k=>$v)
			{
				// traformo il "name" in "id"
				$id	= str_replace(']', '', str_replace('[', '-', $k));

				// se il campo non è readonly
				if(!isset($this->readonly[$id]) || $this->readonly[$id]==0)
				{
					if( $v['type']!='file' && Utility::is_empty($values[$id])  ||
						$v['type']=='file' && empty($_FILES[$id]['name']) )
					{
						$valid = false;
						$this->errors[$id] = 'field_required';
					}
				}
			}
		}

		return $valid;
	}

	/**
	 * Controlla che i field contengano valori conformi a quanto dichiarato
	 * Checks all values in the form against the type of validation defined
	 * @return bool
	 */
	protected function validation()
	{
//		$values = Utility::array_flat($this->values, '', '-');
		$values = $this->values;
		
		if(isset($_SESSION[__CLASS__][$this->uniqid]['validation']))
			$session = $_SESSION[__CLASS__][$this->uniqid]['validation'];
		else
			$session = null;

		$valid = true;
		if(is_array($session))
		{
			foreach($session as $k=>$v)
			{
				// traformo il "name" in "id"
				$id	= str_replace(']', '', str_replace('[', '-', $k));
				
				if( !$this->validate($v['validation'], $values[$id], $error) )
				{
					$valid = false;
					$this->errors[$id] = $error;
				}
			}
		}

		return $valid;
	}

	/**
	 * Valida un valore secondo il tipo dichiato
	 * Valid a value against a type
	 * @param	string	$type
	 * @param	mixed	$value
	 * @param	string	$error
	 * @return	bool
	 */
	protected function validate($type, $value, &$error=null)
	{
		$valid = true;

		$value = trim($value);
		if(empty($value)) return $valid;

		switch($type)
		{
			case 'email':
				if(!preg_match("/^[a-z0-9_\-]+(\.[a-z0-9_\-]+)*@([a-z0-9_\-]+\.)+([a-z]{2,6})$/", $value))
				{
					$valid = false;
					$error = 'email_not_valid';
				}
				break;

			case 'number':
				if(!is_numeric($value))
				{
					$valid = false;
					$error = 'numeric_not_valid';
				}
				break;

			case 'date':
				if(!preg_match("/^(\d{1,2})\/(\d{1,2})\/(\d{4})$/", $value, $matches))
				{
					$valid = false;
					$error = 'date_not_valid';
				}
				else
				{
					$tmp = checkdate($matches[2],$matches[1],$matches[3]);

					if($tmp===false)
					{
						$valid = false;
						$error = 'date_not_valid';
					}
				}
				break;

			default:
				if(is_array($type))
				{
					switch($type['type'])
					{
						case 'equalTo':
							$val = $this->values[strval($type['field'])];

							if($value!=$val)
							{
								$valid = false;
								$error = 'value_mismatch';
							}
							break;
							
						case 'gt':
							$value2 = $this->values[strval($type['field'])];

							if(Utility::isDate($value))
							{
								// confronto di date
								$value  = Utility::date2db($value);
								$value2 = Utility::date2db($value2);
								
								if(Utility::compareDates($value, $value2)!=1)
								{
									$valid = false;
									$error = $this->t->_('valid_error_gt').' "'.$this->t->_($type['field']).'"';
								}
							}
							else
							{
								// confronto di numeri
								if($value>=$value2)
								{
									$valid = false;
									$error = $this->t->_('valid_error_gt').' "'.$this->t->_($type['field']).'"';
								}
							}
							break;
					}
				}
		}
		return $valid;
	}

	/**
	 * Stampa i controlli javascript per validare i valori del form prima di essere postati
	 * Prints javascript code for validation
	 * @return string
	 */
	protected function print_check_js()
	{
		// matrice con gli id dei campi e le validazioni
		$elem = array();
		$mess = array();

		foreach($this->field_required as $name=>$item)
		{
			$elem[$name][] = 'required: true';
		}

		foreach($this->field_validation as $name=>$item)
		{
			foreach($item->validation as $itm)
			{
				if(isset($itm->type))
				{
					// equalTo: "#password"
					// maxlength: 3
					$elem[$name][] = (intval($itm->field)>0) ? $itm->type.': '.$itm->field : $itm->type.': "#'.$itm->field.'"';

					// messaggi personalizzati con traduzioni
					if($itm->type=='gt')
					$mess[$name][] = $itm->type.': "Inserire un valore maggiore di \"'.$this->t->_(strval($itm->field)).'\""';

					if($itm->type=='ge')
					$mess[$name][] = $itm->type.': "Inserire un valore maggiore o uguale di \"'.$this->t->_(strval($itm->field)).'\""';
				}
				else
				{
					$elem[$name][] = $item->validation.': true';
				}
			}
		}

		$rows = array();
		foreach($elem as $key=>$el) $rows[] = '"' . $key . '"' . ': {' . implode(', ', $el) . '}';

		$msgs = array();
		foreach($mess as $key=>$el) $msgs[] = '"' . $key . '"' . ': {' . implode(', ', $el) . '}';

		if(count($rows)>0)
		{
			echo "
					<script type=\"text/javascript\">

					$(function(){

						$('#".$this->xml->id."').validate({
								rules: {
									" . implode(", \n", $rows) . "
								},
								messages: {
									" . implode(', ', $msgs) . "
								},
								errorElement: 'span',
								errorClass: 'form_error',
								submitHandler: function(form) {form.submit();}
							});
					});
					</script>
				";
		}
	}

	/**
	 * Prints extra javascript code (e.g. calendar)
	 * @return string
	 */
	protected function print_extra_js()
	{
		echo "
				<script type=\"text/javascript\">
				<!--
					$(function(){\n";

		$res = $this->xml->xpath('//field[extra="calendar"]');
		if(!empty($res))
		{
			echo "	$('.calendar').datepicker({
							showOn: 'both',
							buttonImage: 'images/calendar.gif',
							buttonImageOnly: true
						}); \n";
		}

		$res = $this->xml->xpath('//field[extra="datemask"]');
		if(!empty($res))
		{
			echo "	$('.datemask').mask('99/99/9999');
					$('.datemask').blur( function(){ checkDate($(this)); } ); \n ";
		}

		$res = $this->xml->xpath('//field[extra="timemask"]');
		if(!empty($res))
		{
			echo "	$('.timemask').mask('99:99');

					$('.timemask').blur(function(){
								data = $(this).val();
								regex = new RegExp('([0-9]+):([0-9]+)','gi');
								data = regex.exec(data);
								if( data != null && data.length==3 )
								{
									if( data[1]<=23 && data[2]<=59 ) return true;
									else
									{
										alert('Ora non valida');
										$(this).focus();
									}
								}
					}); \n";
		}

		$res = $this->xml->xpath('//field[extra="number_format"]');
		if(!empty($res))
		{
			echo "	$('.number_format').blur( function(){ number_format($(this)); } ); \n";
		}

		$res = $this->xml->xpath('//field[extra="tinymce"]');
		if(!empty($res))
		{
/*			echo "	$('textarea.tinymce').tinymce({
						// Location of TinyMCE script
						script_url : 'js/tiny_mce/tiny_mce.js',

						// General options
						theme : 'advanced',
						plugins : 'fullscreen,noneditable',
						noneditable_leave_contenteditable : true,

						// Theme options
						theme_advanced_buttons1 : 'bold,italic,underline,|,bullist,numlist,|,outdent,indent,|,undo,redo,|,fullscreen',
						theme_advanced_buttons2 : '',
						theme_advanced_buttons3 : '',
						theme_advanced_toolbar_location : 'top',
						theme_advanced_toolbar_align : 'left',
						theme_advanced_statusbar_location : 'bottom',
						theme_advanced_resizing : true
					});\n";
*/
			echo "	$('textarea.tinymce').tinymce({
						// Location of TinyMCE script
						script_url : 'js/tiny_mce/tiny_mce.js',

						// General options
						elements : 'nourlconvert',
						convert_urls : false,

						theme : 'advanced',
						plugins : 'fullscreen,noneditable,table,paste,advlink',
						noneditable_leave_contenteditable : true,

						// Theme options
						theme_advanced_buttons1 : 'bold,italic,underline,|,justifyleft,justifycenter,justifyright,justifyfull,|,formatselect,fontselect,fontsizeselect,|,fullscreen',
						theme_advanced_buttons2 : 'cut,copy,paste,pastetext,pasteword,|,undo,redo,|,bullist,numlist,|,outdent,indent,|,link,unlink,|,code',
						theme_advanced_buttons3 : 'tablecontrols',
						theme_advanced_toolbar_location : 'top',
						theme_advanced_toolbar_align : 'left',
						theme_advanced_statusbar_location : 'bottom',
						theme_advanced_resizing : true
					}); \n";
		}

		echo "		});
				-->
				</script>";
	}

	/**
	 * Stampa il form ed eventualmente il codice javascript
	 * Prints the form and javascript code
	 * @param $view		true se il form deve essere stampato in sola lettura
	 * @param $search	true se il form è in modalità ricerca
	 * @return string
	 */
	public function print_form($view=false, $search=false)
	{
		$this->view 	= $view;
		$this->search 	= $search;

		ob_start();

		if(isset($this->xml->method)) $method = $this->xml->method;
		else $method = 'post';

		// onsubmit="return false;"
		echo '<form action="',$this->xml->action,'" method="',$method,'"';

		if(!empty($this->xml->id))
		{
			echo ' id="',$this->xml->id,'" ';
		}
		else
		{
			$this->xml->id = 'form1';
			echo ' id="',$this->xml->id,'" ';
		}

		if(!empty($this->xml->class))
			echo ' class="fb-form ',$this->xml->class,'" ';
		else
			echo ' class="fb-form" ';
		
		if(!empty($this->xml->enctype))	echo ' enctype="',$this->xml->enctype,'" ';

		echo ">\n";

		$j = 0;
		foreach($this->xml->page as $page)
		{
			$quanti = empty($page->repeat) ? 1 : $page->repeat;

			if(isset( $this->repeat[strval($page->id)] ))
			{
				$quanti = $this->repeat[strval($page->id)];
			}

			if($quanti>0)
			{
				if($quanti==1 && empty($page->prefix))
				{
					$this->build_page($page, ++$j);
				}
				else
				{
					for($i=1;$i<=$quanti;$i++)
					{
						$this->build_page($page, ++$j, array('count'=>$i));
					}
				}
			}
		}

		// un campo nascosto per riconoscere quale form è stato postato, solo quando è in edit
		if(!$this->search)
		{
			echo '<input type="hidden" id="uniqid" name="uniqid" value="'.$this->uniqid.'" />';
		}

		echo "</form>\n";
//		echo '<script type="text/javascript">$(".page:visible:last").css("page-break-after","auto");</script>';

		// controllo js degli errori ed extra solo quando si è in edit/add
		// oppure stampa di tinymce se presente in view
		$tiny = $this->xml->xpath('//field[extra="tinymce"]');
		if(!$this->view)
		{
			if(!$this->search) $this->print_check_js();
			$this->print_extra_js();
		}
		elseif(!empty($tiny))
		{
			$this->print_extra_js();
		}

		// se c'è almeno un campo readonly imposto la scadenza della validità
/*		if(isset($_SESSION[__CLASS__][$this->uniqid]['readonly']))
		{
			$_SESSION[__CLASS__][$this->uniqid]['readonly']['zz_expire'] = time() + $this->expiration;
		}
*/
		// metto in sessione la struttura dei campi obbligatori
//		$_SESSION[__CLASS__][$this->uniqid]['required'] = $this->field_required;
		$_SESSION[__CLASS__][$this->uniqid]['required'] = Utility::simplexml2array($this->field_required);
//		$_SESSION[__CLASS__][$this->uniqid]['required']['zz_expire'] = time() + $this->expiration;

		// metto in sessione la struttura dei campi da validare
		$_SESSION[__CLASS__][$this->uniqid]['validation'] = Utility::simplexml2array($this->field_validation);
//		$_SESSION[__CLASS__][$this->uniqid]['validation']['zz_expire'] = time() + $this->expiration;
		
		
		$_SESSION[__CLASS__][$this->uniqid]['expire'] = time() + $this->expiration;
		
		return ob_get_clean();
	}

	protected function build_page($page, $progre, $param=array())
	{
		if(!empty($page->id))
		{
			$c = !empty($param['count']) ? '-'.$param['count'] : '';
			
			$id = ' id="'.$page->id.$c.'" ';
		}
		else
		{
			$id = ' ';
		}
		
		echo '<div'.$id.'class="page '.$page->class.'" >'. "\n";

		// print caption if present
		if(!empty($page->caption))
		{
			// substitute placeholder in caption
			$caption = $page->caption;
			if(!empty($param['count']))
			{
				$caption = sprintf($caption, $param['count']);
			}
			echo '<span class="legend">',$caption,'</span>', "\n";
		}

		$prefix = !empty($page->prefix) ? $page->prefix . '['.$param['count'].']' : '';

		$j = 1;
		foreach($page->section as $section) $this->build_section($section, 'page-'.$progre, $j++, $prefix);

		echo "</div>\n";
	}

	protected function build_section($section, $page_prog, $sect_prog, $prefix='', $param=array())
	{
		$class = !empty($section->class) ? ' class="'.$section->class.'" ' : '';

		if(!empty($section->id))
			$id = strval($section->id).'-'.$sect_prog;
		else
			$id = 'section-'.$sect_prog;
		
		$id = $page_prog.'-'.$id;
		
		// anche la sezione stessa può avere un prefisso
		if( !empty($section->prefix) )
		{
			// aggiungo questo prefisso solo se la section è singola,
			// altrimenti in quelle ripetute viene scritto due volte
			if(empty($param['count']))
			{
				$prefix = !empty($prefix) ? $prefix.'['.$section->prefix.']' : $section->prefix;
			}
		}

//		echo '<div id="'.$id.'" class="fieldset '.$section->class.' '.$section->prefix.'" >', "\n";
		echo '<div id="'.$id.'" class="section '.$section->class.' '.$section->prefix.'" >', "\n";

		if(!empty($section->caption))
		{
			$caption = $section->caption;
			if(!empty($param['count']))
			{
				$caption = $this->t->_(sprintf($caption, $param['count']));
			}
			echo '<span class="legend">',$caption,'</span>', "\n";
		}

		// print fields
		foreach($section->field as $field)
		{
			if($this->search)
				$this->build_field_search($field,$prefix);
			else
				$this->build_field($field,$prefix);
		}

		// print children sections
		$j = 1;
		foreach($section->section as $s2)
		{
			$quanti = empty($s2->repeat) ? 1 : $s2->repeat;
			
			// queste 2 righe servono per comandare in run-time il numero di box stampati
			if(empty($prefix))
				$tmp = strval($s2->id);
			else
				$tmp = str_replace(']','',str_replace('[','-',$prefix)) . '-' . strval($s2->id);
			
			if(isset( $this->repeat[$tmp] )) $quanti = $this->repeat[$tmp];

			if($quanti==1 && empty($s2->prefix))
			{
				$this->build_section($s2, $id, $j++, $prefix);
			}
			else
			{
				for($i=1;$i<=$quanti;$i++)
				{
					if(empty($prefix))
					{
						$tmp = strval($s2->prefix).'['.$i.']';
					}
					else
					{
						$tmp = $prefix.'['.strval($s2->prefix).']['.$i.']';
					}
					$this->build_section($s2, $id, $j++, $tmp, array('count'=>$i));
				}
			}
		}

//		echo "<div class=\"float_none clear_both\"></div></div>\n";
		echo "</div>\n";
	}

	/**
	 *
	 * @param	SimpleXMLElement	$field
	 * @param	string				$prefix  prefisso per l'id e il nome del campo
	 * @return	void
	 */
	protected function build_field($field, $prefix='')
	{
		// insert the prefix if present
		if(!empty($prefix))
		{
			$id = str_replace(']','',str_replace('[','-',$prefix)) . '-' . strval($field->id);

			if(!empty($field->name))
			{
				$name = $prefix . '[' . strval($field->name) . ']';
			}
			else
			{
				$name = $prefix . '[' . strval($field->id) . ']';
			}

			// $prefix = vett[1]
			// $name   = vett[1][indirizzo]
			// $id     = vett-1-indirizzo
		}
		else
		{
			$id = strval($field->id);

			if(!empty($field->name))
			{
				$name = $field->name;
			}
			else
			{
				$name = $id;
			}
		}

		$extra = '';
		if(isset($field->extra))
		{
			$extra = $field->extra;
		}

		if(isset($field->readonly))
		{
			$this->readonly[$id] = 1;
		}

		// campi obbligatori
		if(isset($field->required))
		{
			$this->field_required[$name] = $field;
		}

		// campi con validazione
		if(isset($field->validation))
		{
			$this->field_validation[$name] = $field;
		}

		$edit = !$this->view && empty($this->readonly[$id]);

		// prendo la label se impostata, altrimenti l'id
		$label = isset($field->label) ? strval($field->label) : strval($field->id);

		// traduzione
		if(isset($this->t)) $label = $this->t->_($label);

		// asterisco per campo obbligatorio
		$req = $field->required==1 ? ' *' : '';
		$label = $label.$req;

		/* scrive il div quando:
		 * - è un bottone e si è in edit/add
		 * - non è un hidden nè un bottone
		 *
		 * serve per non stampare il div quando si è in view e c'è un campo hidden
		 */
		$show_div = ($field->type=='button' && $edit) || 
					($field->type!='hidden' && $field->type!='csrf' && $field->type!='button');
		
		if( $show_div )
		{
			if(!empty($field->class))
				$class = 'class="fb-div '.$field->class.'"';
			else
				$class = 'class="fb-div"';

			echo '<div id="',str_replace(':','_',$id),'-div" ',$class,' >';

			switch($field->type)
			{
				case 'radio':
				case 'multicheck':
					echo '<label>',$label,'</label>';
					break;

				case 'button':
					break;

				default:
					echo '<label for="',$id,'">',$label,'</label>';
			}
		}
		
		switch ($field->type)
		{
			case 'button':

				if($edit)
				{
					$this->print_button($field, $id, $label);
				}
				break;

			case 'textarea':

				if($edit)
				{
					$default = isset($this->values[$id]) ? $this->values[$id] : null;
					$this->print_textarea($field, $id, $name, $default, $extra);
				}
				else
				{
					if($extra=='tinymce')
					{
						echo '<textarea id="',$id,'" name="',$name,'" class="',$extra,'" >';
						if( isset($this->values[$id]) )
						{
							echo '<div class="mceNonEditable">' . $this->values[$id] . '</div>';
						}
						echo '</textarea>';
					}
					else
					{
						echo '<div class="fb-textarea-ro" id="',$id,'">';
						if( isset($this->values[$id]) ) echo $this->values[$id];
						echo '</div>';
					}
					
					$this->print_hidden($name, $this->values[$id]);
				}
				break;

			case 'select':

				if($edit)
				{
					$default = isset($this->values[$id]) ? $this->values[$id] : null;
					$this->print_select($field, $id, $name, $default);
				}
				else
				{
					echo '<div class="select fb-select-ro" id="',$id,'">';

					if (!empty($field->lookup))
					{
						$where = is_numeric($this->values[$id]) ? $this->values[$id] : "'".$this->values[$id]."'";

						$q1 =	'SELECT ' . $field->lookup->id . ' AS id, ' 
										  . $field->lookup->caption . ' AS caption ' .
								'FROM ' . $field->lookup->table . ' ' .
								'WHERE '.$field->lookup->id.' = '.$where;
						$row = $this->db->fetchRow($q1);

						if( intval($field->lookup->showId)==1 )
						{
							echo $row['id'] . ' - ' . $row['caption'];
						}
						else
						{
							echo $row['caption'];
						}
					}

					// scorre i valori statici da XML
					if (!empty($field->values->opt))
					{
						foreach($field->values->opt as $opt)
						{
							if( isset($this->values[$id]) && $this->values[$id] == $opt->value )
							{
								echo $this->t->_(strval($opt->label));
							}
						}
					}

					// scorre i valori dinamici
					if(isset($this->selects[$id]))
					{
						foreach($this->selects[$id] as $key=>$val)
						{
							if( isset($this->values[$id]) && $this->values[$id] == $key )
							{
								echo $this->t->_($val);
							}
						}
					}
					echo '</div>';
					
					$this->print_hidden($name, $this->values[$id]);
				}
				break;

			case 'radio':

				if (!empty($field->values->opt))
				{
					$value = isset($this->values[$id]) ? $this->values[$id] : strval($field->value);

					echo '<ul class="inner_radio">';
					foreach($field->values->opt as $opt)
					{
						$opt_id = $id.'-'.$opt->value;
						echo '<li>';
						echo '<input type="',$field->type,'" id="',$opt_id,'" name="',$name,'" class="radio" ';
						echo 'value="',$opt->value,'"';

						if( isset($value) && $value == strval($opt->value) )
						{
							echo ' checked="checked" ';
						}

						if( !$edit ) echo ' disabled="disabled" ';

						echo ' />';
						
						echo '<label for="',$opt_id,'">',$this->t->_(strval($opt->label)),'</label>';
						echo '</li>';
					}
					echo '</ul>';
					
					if( !$edit ) $this->print_hidden($name, $value);
				}
				break;

			case 'checkbox':

				echo '<input type="',$field->type,'" id="',$id,'" name="',$name,'"';
				echo ' class="checkbox" ';
				echo 'value="',$field->value,'"';

				if( isset($this->values[$id]) && $this->values[$id] == $field->value )
				{
					echo ' checked="checked" ';
				}

				if( !$edit ) echo ' disabled="disabled" ';

				echo ' />';
				
				if( !$edit ) $this->print_hidden($name, $this->values[$id]);
				break;

			case 'multicheck':

				if (!empty($field->lookup))
				{
					// ANDREA 23/08/2010 categorizzazione delle voci
					if(isset($field->lookup->category))
					{
						$q1 = 'SELECT ' . $field->lookup->id . ' AS id, '
							            . $field->lookup->caption . ' AS caption, '
							            . $field->lookup->category . ' AS category ' .
							  'FROM ' . $field->lookup->table . ' ';
					}
					else
					{
						$q1 = 'SELECT ' . $field->lookup->id . ' AS id, '
						                . $field->lookup->caption . ' AS caption ' .
							  'FROM ' . $field->lookup->table . ' ';
					}

					if(!empty($field->lookup->where))
					{
						$q1 .= 'WHERE ' . $field->lookup->where;
					}
					if(!empty($field->lookup->orderby))
					{
						$q1 .= ' ORDER BY ' . $field->lookup->orderby;
					}
					else
					{
						$q1 .= ' ORDER BY id ASC';
					}

					$res = $this->db->fetchAll($q1);
				}

				// prende i valori aggiunti dinamicamente
				if(isset($this->selects[$id]))
				{
					$res = $this->selects[$id];
				}

				if(!empty($res))
				{
					echo '<ul class="multicheck_container">';

					if(isset($field->lookup->category))
					{
						$first = true;
						$prec = '';
						foreach ($res as $row)
						{
							$key = array_keys($row);

							// MODIFICA ANDREA 26/06/2009
							// mostro solo le righe con i check selezionati
							$id2 	= $id . '-' . $row[$key[0]];
							
							$values = isset($this->values[$id2]) ? $this->values[$id2] : false;

							if( $values || !$this->view)
							{
								// categoria
								if( $prec!=$row[$key[2]] )
								{
									$prec = $row[$key[2]];

									if($first)
										$first = false;
									else
										echo '</ul></li>';
									
									echo '<li class="catego"><span>'.$row[$key[2]].'</span><ul>';
								}

								$name2 	= $name . '[' . $row[$key[0]] . ']';

								echo '<li id="',$id2,'_div" class="multicheck">';

								echo '<input type="checkbox" id="',$id2,'" name="',$name2,'"';
								echo ' class="checkbox" value="1" ';

								if( $values == 1 ) echo ' checked="checked" ';

								if( !$edit ) echo ' disabled="disabled" ';

								echo ' />';

								echo '<label for="'.$id2.'">'.Utility::htmlutf($row[$key[1]]).'</label>';

								echo '</li>';
							}
						}
						echo '</ul></li>';
					}
					else
					{
						foreach ($res as $row)
						{
							$key = array_keys($row);

							// MODIFICA ANDREA 26/06/2009
							// mostro solo le righe con i check selezionati
							$id2 	= $id . '-' . $row[$key[0]];
							
							$values = isset($this->values[$id2]) ? $this->values[$id2] : false;
							
							if( $values || !$this->view)
							{
								$name2 	= $name . '[' . $row[$key[0]] . ']';

								echo '<li>';

								echo '<input type="checkbox" id="',$id2,'" name="',$name2,'"';
								echo ' class="checkbox" value="1" ';

								if( $values == 1 ) echo ' checked="checked" ';

								if( !$edit ) echo ' disabled="disabled" ';

								echo ' />';

								echo '<label for="'.$id2.'">'.Utility::htmlutf($row[$key[1]]).'</label>';
								
								if( !$edit ) $this->print_hidden($name2, $values);

								echo '</li>';
							}
						}
					}
					echo '</ul>';
				}
				break;

			case 'span':
				$this->print_span($field, $id);
				break;

			case 'csrf':
//				$this->checkCsrf = true;
				break;

			default:

				$value = isset($this->values[$id]) ? $this->values[$id] : strval($field->value);

				if($edit)
				{
					$this->print_input($field, $id, $name, $value, $extra);
				}
				else
				{
					if($field->type!='hidden')
					{
						echo '<div class="input fb-input-ro" id="',$id,'">';
						if(isset($value)) echo $value;
						echo '</div>';
					}
					
					$this->print_hidden($name, $value);
				}
		}

		// 09/10/2009 MODIFICA se il campo è in readonly lo metto in sessione
/*		if(!$edit)
		{
			$_SESSION[__CLASS__][$this->uniqid]['readonly'][$id] = strval($this->values[$id]);
		}
*/
		// elemento che contiene la descrizione degli errori
		if( !empty($this->errors[$id]) )
		{
			$error = isset($this->t) ? $this->t->_($this->errors[$id]) : $this->errors[$id];
			echo '<span class="form_error">',$error,'</span>';
		}

		if( $show_div )
		{
			echo '</div>', "\n";
		}
	}

	protected function build_field_search($field, $prefix='')
	{
		$count = count($field->search->op);

		if($count>0 || $field->type=='button' || $field->type=='span')
		{
			// insert the prefix if present
			if(!empty($prefix))
			{
				$id = $prefix . '-' . strval($field->id);
			}
			else
			{
				$id = strval($field->id);
			}

			$extra = '';
			if(isset($field->extra))
			{
				$extra = $field->extra;
			}

			// prendo la label se impostata, altrimenti l'id
			$label = isset($field->label) ? strval($field->label) : strval($field->id);

			// traduzione
			if(isset($this->t)) $label = $this->t->_($label);

			if(!empty($field->class))
				$class = 'class="fb-div '.$field->class.'"';
			else
				$class = 'class="fb-div"';

			echo '<div id="',str_replace(':','_',$id),'-div" ',$class,' >';

			if($field->type!='button') echo '<label>',$label,'</label>';

			switch ($field->type)
			{
				case 'button':

					$this->print_button($field, $id, $label);
					break;

				case 'textarea':

					$this->print_textarea($field, $id, $id, $this->values[$id]);
					break;

				case 'select':

					$i = 0;
					echo '<div class="fb-param">';
					foreach($field->search->op as $op)
					{
						echo '<div>';
						$this->print_op(strval($op), strval($field->id), $i);

						$default = isset($this->values[$id][$i]['val']) ? $this->values[$id][$i]['val'] : null;
						
						$this->print_select($field, $id.'-'.$i, $id.'['.$i.'][val]', $default);

						$i++;

						if($i<$count)
						{
							$s1 = $this->values[$id][$i]['con'] == 'and' ? ' selected="selected" ' : '';
							$s2 = $this->values[$id][$i]['con'] == 'or' ? ' selected="selected" ' : '';

							echo '<select class="frm_operator_con" name="'.$id.'['.$i.'][con]">
									<option value="and"'.$s1.'>'.$this->t->_('AND').'</option>
									<option value="or"'.$s2.'>',$this->t->_('OR'),'</option>
								 </select>';
						}
						echo '</div>';
					}
					echo '</div>';
					break;

				case 'radio':

					if (!empty($field->values->opt))
					{
						$i = 0;

						//$value = !empty($this->values[$id][$i]['val']) ? $this->values[$id][$i]['val'] : strval($field->value);
						$value = !Utility::is_empty($this->values[$id][$i]['val']) ? $this->values[$id][$i]['val'] : strval($field->value);

						$op = strval($field->search->op[$i]);
						echo '<div class="fb-param">';
						$this->print_op(strval($op), strval($field->id), $i);

						echo '<ul class="inner_radio">';
						foreach($field->values->opt as $opt)
						{
							$opt_id = $id.'-'.$opt->value;

							echo '<li>';
							echo '<input type="',$field->type,'" id="',$opt_id,'" name="',$id.'['.$i.'][val]','" class="radio" ';
							echo 'value="',$opt->value,'"';

							if( isset($value) && $value == $opt->value )
							{
								echo ' checked="checked" ';
							}

							echo ' />';
							echo '<label for="',$opt_id,'">',$opt->label,'</label>';
							echo '</li>';
						}
						echo '</ul>';
						echo '</div>';
					}
					break;

				case 'checkbox':

					$i = 0;
					$op = strval($field->search->op[$i]);
					echo '<div class="fb-param">';
					$this->print_op(strval($op), strval($field->id), $i);

					echo '<input type="',$field->type,'" id="',$id,'" name="',$id.'['.$i.'][val]','"';
					echo ' class="checkbox" ';
					echo 'value="',$field->value,'"';

					if( isset($this->values[$id][$i]['val']) && $this->values[$id][$i]['val'] == $field->value )
					{
						echo ' checked="checked" ';
					}

					echo ' />';
					echo '</div>';
					break;

				case 'span':
					$this->print_span($field, $id);
					break;

				default:

					$i = 0;
					echo '<div class="fb-param">';
					foreach($field->search->op as $op)
					{
						$value = !empty($this->values[$id][$i]['val']) ? $this->values[$id][$i]['val'] : strval($field->value);

						echo '<div>';
						$this->print_op(strval($op), strval($field->id), $i);

						$this->print_input($field, $id.'-'.$i, $id.'['.$i.'][val]', $value, $extra);

						$i++;

						// mostro gli operatori AND e OR
						if($i<$count)
						{
							$tmp = isset($this->values[$id][$i]['con']) ? $this->values[$id][$i]['con'] : null;
							
							$s1 = $tmp == 'and' ? ' selected="selected" ' : '';
							$s2 = $tmp == 'or'  ? ' selected="selected" ' : '';

							echo '<select class="frm_operator_con" name="'.$id.'['.$i.'][con]">
									<option value="and"'.$s1.'>'.$this->t->_('AND').'</option>
									<option value="or"'.$s2.'>',$this->t->_('OR'),'</option>
								 </select>';
						}
						echo '</div>';
					}
					echo '</div>';
			}

			echo '</div>', "\n";
		}
	}

	protected function print_op($op, $id, $i)
	{
		$all_num['eq'] = '=';
		$all_num['gt'] = '>';
		$all_num['ge'] = '>=';
		$all_num['lt'] = '<';
		$all_num['le'] = '<=';
		$all_num['not'] = '<>';

		$all_str['like'] 		= 'LIKE';
		$all_str['not_like']	= 'NOT LIKE';

		$op = explode(',',$op);
		
		if(count($op)==1 && $op[0]!='all_num' && $op[0]!='all_str') 
		{
			$class = ' hidden';
		}
		else
		{
			$class = '';
		}

		echo '<select class="frm_operator'.$class.'" name="'.$id.'['.$i.'][op]">';

		foreach($op as $oper)
		{
			switch(trim($oper))
			{
				case 'all_num':

					foreach($all_num as $key=>$val)
					{
						echo '<option value="'.$key.'"';
						if(isset($this->values[$id][$i]['op']) && $this->values[$id][$i]['op']==$key) echo ' selected="selected" ';
						echo '>'.Utility::htmlutf($val).'</option>';
					}
					break;

				case 'all_str':

					foreach($all_str as $key=>$val)
					{
						echo '<option value="'.$key.'"';
						if(isset($this->values[$id][$i]['op']) && $this->values[$id][$i]['op']==$key) echo ' selected="selected" ';
						echo '>'.Utility::htmlutf($val).'</option>';
					}
					break;

				default:

					if(array_key_exists($oper, $all_str))
					$arr = $all_str;
					else
					$arr = $all_num;

					echo '<option value="'.$oper.'"';
					if(isset($this->values[$id][$i]['op']) && $this->values[$id][$i]['op']==$oper) echo ' selected="selected" ';
					echo '>'.Utility::htmlutf($arr[$oper]).'</option>';
			}
		}

		echo '</select>';
	}

	protected function print_button($field, $id, $label)
	{
		echo '<button id="',$id,'" type="',$field->type2,'" class="button">', $label, '</button>';
	}

	protected function print_textarea($field, $id, $name, $value, $class='')
	{
		echo '<textarea id="',$id,'" name="',$name,'" class="',$class,'" rows="4" cols="50">';
		if( isset($value) ) echo $value;
		echo '</textarea>';
	}

	protected function print_input($field, $id, $name, $value, $class='')
	{
		$class .= ' input';
		
		echo '<input type="'.$field->type.'" id="'.$id.'" name="'.$name.'"';
		if( isset($class) ) echo ' class="'.$class.'" ';
		if( isset($value) ) echo ' value="'.Utility::htmlutf($value).'" ';
		echo ' />';
	}

	/**
	 * Prints a select (combobox) element
	 *
	 * @param	SimpleXMLElement	$field
	 * @param	int					$id			id of the element
	 * @param	string				$name		name of the element
	 * @param	mixed				$default	default value of the element
	 * @return	string
	 */
	protected function print_select($field, $id, $name, $default)
	{
		$data = array();
		$t = $this->t;

		$default = !empty($default) ? $default : strval($field->value);

		echo '<select id="',$id,'" name="',$name,'">';

		// scorre i valori fissi da XML
		if (!empty($field->values->opt))
		{
			foreach($field->values->opt as $opt)
			{
				$data[strval($opt->value)] = strval($opt->label);
			}
		}

		// scorre i valori della query
		if (!empty($field->lookup))
		{
			if(isset($field->lookup->category))
			{
				$q1 = 'SELECT ' . $field->lookup->id . ' AS id, '
								. $field->lookup->caption . ' AS caption, '
								. $field->lookup->category . ' AS category ' .
					  'FROM ' . $field->lookup->table . ' ';
			}
			else
			{
				$q1 = 'SELECT ' . $field->lookup->id . ' AS id, '
								. $field->lookup->caption . ' AS caption ' .
					  'FROM ' . $field->lookup->table . ' ';
			}

			if(!empty($field->lookup->where))
			{
				$q1 .= 'WHERE ' . $field->lookup->where;
			}
			if(!empty($field->lookup->group))
			{
				$q1 .= 'GROUP BY ' . $field->lookup->group;
			}
			if(!empty($field->lookup->orderby))
			{
				$q1 .= ' ORDER BY ' . $field->lookup->orderby;
			}
			else
			{
				$q1 .= ' ORDER BY id ASC';
			}
			
			$res = $this->db->fetchAll($q1);
			
			foreach($res as $row)
			{
				if(!empty($row['category']))
				{
					$data[$row['category']][$row['id']] = $row['caption'];
				}
				else
				{
					$data[$row['id']] = $row['caption'];
				}
			}
		}

		// appendo i valori aggiunti dinamicamente
		if(isset($this->selects[$id]))
		{
			$data += $this->selects[$id];
		}

		foreach($data as $key=>$val)
		{	
			if(is_array($val))
			{
				echo '<optgroup label="'.$key.'">';

				foreach($val as $k=>$v)
				{
					if( $default == $k )
						$selected = true;
					else
						$selected = false;

					if( intval($field->lookup->showId)==1 )
					{
						$v = $k.' - '.$t->_($v);
					}
					else
					{
						$v = $t->_($v);
					}

					$this->print_option($k, $v, $selected);
				}

				echo '</optgroup>';
			}
			else
			{
				if( $default == $key )
					$selected = true;
				else
					$selected = false;

				if( intval($field->lookup->showId)==1 )
				{
					$val = $key.' - '.$t->_($val);
				}
				else
				{
					$val = $t->_($val);
				}

				$this->print_option($key, $val, $selected);
			}
		}
		echo '</select>';
	}
	
	protected function print_option($key, $val, $selected=false)
	{
		echo '<option value="'.$key.'"';

		if($selected) echo ' selected="selected" ';

		echo '>';

		echo $val;

		echo '</option>';
	}

	protected function print_span($field, $id)
	{
		if(isset($field->class))
			$class = ' class="'.$field->class.'"';
		else
			$class = '';

		echo '<span id="',$id,'"'.$class.'>';
		echo $field->value;
		echo '</span>';
	}
	
	protected function print_hidden($name, $value = null, $attribs = null)
	{
		echo '<input type="hidden" name="'.$name.'" value="'.$this->escape($value).'" />';
	}
	
	protected function escape($var)
	{
		// Convert special characters to HTML entities
		return htmlspecialchars($var);
		
		// Convert all applicable characters to HTML entities
//		return htmlentities($var);
	}
}
