<?php
/**
 * This class contains some utility functions.
 *
 * @package		FormBuilder
 * @author		Andrea Venturi
 * @copyright	Copyright (c) 2009-2012 Andrea Venturi
 * @license		http://www.gnu.org/licenses/lgpl.html	GNU Lesser General Public License
 * @version		0.1
 */
class Utility
{
	/**
	 * Tenta la conversione di una stringa UTF8 in entities HTML.
	 * @param	string	$str	stringa da codificare
	 * @return	string	stringa codificata in HTML
	 */
	public static function htmlutf($str)
	{
		$ret = '';

		$str = trim($str);
		$ret = htmlentities($str,ENT_COMPAT,'UTF-8');
		if(strlen($ret)<1) $ret = htmlentities($str);

		return $ret;
	}

	/**
	 * Converte un oggetto SimpleXMLElement in array
	 * http://jamesroberts.name/blog/2010/02/10/simplexml-to-array/
	 * http://theserverpages.com/php/manual/en/ref.simplexml.php
	 *
	 * @param	SimpleXMLElement  $xml
	 * @return  array
	 */
	public static function simplexml2array($xml)
	{
		$x = null;

		if (@get_class($xml) == 'SimpleXMLElement')
		{
			$attributes = $xml->attributes();
			foreach($attributes as $k=>$v)
			{
				if ($v) $a[$k] = (string) $v;
			}
			$x = $xml;
			$xml = get_object_vars($xml);
		}

		if (is_array($xml))
		{
			if (count($xml) == 0) return (string) $x; // for CDATA
			foreach($xml as $key=>$value)
			{
				$r[$key] = self::simplexml2array($value);
			}
			if (isset($a)) $r['@attributes'] = $a;    // Attributes
			return $r;
		}
		return (string) $xml;
	}

	/**
	 * Trasforma un array multidimensionale in uno monodimensionale.
	 * Esempio:	test[a][1][b]	=>	test[a_1_b]
	 * @param	array	$array	array multidimensionale
	 * @param	string	$prefix	prefisso aggiunto al nome dei campi
	 * @param	string	$sep	carattere separatore tra i nomi delle chiavi
	 * @return	array	array monodimensionale
	 */
	public static function array_flat(array $array, $prefix='', $sep='_')
	{
		$ret = array();
		foreach($array as $key=>$val)
		{
			if(is_array($val))
			{
				$arr = self::array_flat($val, $prefix.$key.$sep, $sep);
				$ret = array_merge($ret, $arr);
			}
			else
			{
				$ret[$prefix.$key] = $val;
			}
		}
		return $ret;
	}

	/**
	 * Funzione molto simile a "bool empty(mixed $var)".
	 * L'unica differenza  che con il valore numerico "0" viene restituito FALSE.
	 * @param	mixed	$string		variabile da controllare
	 * @return	bool	vedi sopra
	 */
	public static function is_empty($var)
	{
		$var = trim($var);
		if(!is_numeric($var)) return empty($var);

		return false;
	}

	/**
	 * Controlla se una stringa contiene una data in un certo formato
	 * @param	string	$date	stringa da validare
	 * @param	string	$format	il formato della data
	 * @return	bool	true se rispetta il formato
	 */
	public static function isDate($date, $format='ita')
	{
		switch($format)
		{
			// formati validi: 23/11/2009, 23.11.2009, 23-11-2009, 4/6/09
			case 'ita':
				if(empty($date)) return false;
				if(preg_match('/^([0-9]{1,2})[\/\.-]([0-9]{1,2})[\/\.-]([0-9]{2,4})$/',$date,$matches))
				{
					// controllo che il giorno e il mese siano nei range
					if($matches[1]<1 || $matches[1]>31) return false;
					if($matches[2]<1 || $matches[2]>12) return false;

					return true;
				}
				break;

				// formati validi: 23/11/2009 14:25:16, 4-6-09 14:25:16
			case 'datetime':
				if(empty($date)) return true;
				if(preg_match('/^([0-9]{1,2})[\/\.-]([0-9]{1,2})[\/\.-]([0-9]{2,4}) ([0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2})$/',$date,$matches)) return true;
				break;

				// formati validi: 2009-11-23
			case 'db':
				if(empty($date)) return true;
				if(preg_match('/^([0-9]{4})-([0-9]{2})-([0-9]{2})$/',$date,$matches)) return true;
				break;
		}
		return false;
	}

	/**
	 * Prende una data in formato italiano (dd/MM/YYYY, dd-MM-YYYY, dd.MM.YYYY)
	 * e lo ritorna in formato DATETIME YYYY-MM-dd
	 * @param	string	$date	la stringa contenente la data
	 * @return	string	la data nel nuovo formato
	 */
	public static function date2db($date)
	{
		if(empty($date)) return null;

		if(preg_match('/^([0-9]{1,2})[\/\.-]([0-9]{1,2})[\/\.-]([0-9]{2,4})$/',$date,$matches))
		{
			if ($matches[3] < 100) $matches[3] += 1900;

			$date = $matches[3].'-'. $matches[2].'-'.$matches[1];
			return $date;
		}
		elseif(preg_match('/^([0-9]{4})-([0-9]{2})-([0-9]{2})$/',$date,$matches))
		{
			return $date;
		}
		return null;
	}

	/**
	 * Compares $dateA against $dateB
	 * @param	string	$dateA	first date
	 * @param	string	$dateB	second date
	 * @return	int		0 = dates are equal, 1 = $dateA is greater, 2 = $dateB is greater
	 */
	public static function compareDates($dateA, $dateB)
	{
		$dateA = strtotime($dateA);
		$dateB = strtotime($dateB);

		if($dateA>$dateB) return 1;
		if($dateA<$dateB) return 2;

		return 0;
	}
}
